using System;
using System.IO;

namespace Cryptura
{
    public class WXFile
    {
        public string Path { get; set; }

        public WXFile(string path)
        {
            this.Path = path;
        }

        public void CreateFromPassword(Password password)
        {
            this.WriteHeader();
            this.SetName(password.Name);
            this.SetValue(password.Value);
        }

        public bool Verify()
        {
            try
            {
                var lines = File.ReadAllLines(this.Path);

                if (lines.Length < 3)
                    return false;

                if (lines[0] != "##WXFile##")
                    return false;

                if (string.IsNullOrWhiteSpace(lines[1]) || string.IsNullOrWhiteSpace(lines[2]))
                    return false;

                return true;
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Error reading file: {ex.Message}");
                return false;
            }
        }

        public string GetName()
        {
            try
            {
                var lines = File.ReadAllLines(this.Path);
                return lines.Length >= 2 ? lines[1] : "";
            }
            catch
            {
                return "";
            }
        }

        public void SetName(string value)
        {
            try
            {
                var lines = File.ReadAllLines(this.Path);

                if (lines.Length < 3)
                    Array.Resize(ref lines, 3);

                lines[1] = value;
                File.WriteAllLines(this.Path, lines);
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Error writing name: {ex.Message}");
            }
        }

        public string GetValue()
        {
            try
            {
                var lines = File.ReadAllLines(this.Path);
                return lines.Length >= 3 ? EncryptionLibrary.Decrypt(MainWindow.MasterPassword, lines[2]) : "";
            }
            catch
            {
                return "";
            }
        }

        public void SetValue(string value)
        {
            try
            {
                string encryptedHash = EncryptionLibrary.Encrypt(MainWindow.MasterPassword, value);
                
                var lines = File.ReadAllLines(this.Path);

                if (lines.Length < 3)
                    Array.Resize(ref lines, 3);

                lines[2] = encryptedHash;
                File.WriteAllLines(this.Path, lines);
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Error writing value: {ex.Message}");
            }
        }
        
        public void WriteHeader()
        {
            try
            {
                var lines = File.Exists(this.Path) ? File.ReadAllLines(this.Path) : new string[0];

                if (lines.Length < 1)
                    Array.Resize(ref lines, 1);

                lines[0] = "##WXFile##";

                File.WriteAllLines(this.Path, lines);
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Error writing header: {ex.Message}");
            }
        }
    }
}
